function [y,filt]=nt_destep(x,thresh,filt,guard,depth);
%[y,B,A]=nt_destep(x,thresh,filt,guard,depth) - remove step glitch from MEG data
%
%  y: clean data
%  filt: estimated filter coefficients (filt={B,A})
%
%  x: data to clean (time * channels)
%  thresh: threshold for variance reduction [default: 0.1]
%  filt: antialiasing filter, if [] estimate [default:[]]
%  guard: minimum duration of stable interval in samples [default: 1000]
%  depth: recursion depth for nt_split [default:6], determines number of steps
%
% Searches for large steps, removes them if variance ratio less than 
% "thresh", and both intervals shorter than "guard".
%
% Parameter "filt" can be:
%  []: estimate from the data
%  {B,A}: filter coefficients
%  file name: mat file containg B and A
% 
% Estimation from data might sometimes give erroneous results. Consider running
% nt_destep on a clean step, save the filter estimates, and use them.
% 
% See also nt_detrend_robust.
%
% NoiseTools.
nt_greetings;

if nargin<1; error('!'); end
if nargin<2||isempty(thresh); thresh=0.1; end
if nargin<3; filt=[]; end
if nargin<4||isempty(guard); guard=1000; end
if nargin<5 || isempty(nglitch); depth=6; end

if isempty(filt)
    if ~6==exist('stmcb'); 
        error('Need function stmcb from signal toolbox (or nlinfit from statistics toolbox)');
    end
end
if ischar(filt)
    load(filt); % should contain B and A
    if ~1==exist('A') || ~1==exist('B');
        error(['file >',filt,'< does not contain B and A']);
    end
    filt={B,A};
end  

y=x;

nfit=50; % number of samples for IR fit
B=1; A=1; % avoid error if no step
%if nfit*2>guard; error('!'); end

for iChan=1:size(x,2)
    % find step indices
    [splitI,score_vector,score]=nt_split(x(:,iChan),depth,thresh,guard);
    
    if ~isempty(splitI)
        splitI=splitI-5;
        splitI=[0,splitI,size(x,1)];
        for iSplit=2:numel(splitI)-1
            m1=mean(y(splitI(iSplit-1)+1:splitI(iSplit),iChan));
            m2=mean(y(splitI(iSplit)+1:splitI(iSplit+1),iChan));
            step=(m2-m1);
            step_response=y(splitI(iSplit):splitI(iSplit+1),iChan);
            if size(step_response,1)>nfit; step_response=step_response(1:nfit); end
            step_response=[step_response;ones(size(step_response,1),1)*m2]; % helps ensure stable filter
%             step_response=y(splitI(iSplit)+(1:nfit),iChan);
%             step_response=[step_response;ones(nfit,1)*m2]; % helps ensure stable filter

            %figure(1); clf; plot(step_response); pause
            
            if isempty(filt)  % estimate filter by fitting step response    
                %{ 
                The filter is estimated anew for each step and each
                channel.
                %}
                xx=ones(size(step_response,1),1)*step; yy=[step_response];
                if 1
                    [B,A]=stmcb(diff([step_response]/step),10,10);
                else
                    BETA0=[1,zeros(1,5),1,zeros(1,5)];
                    fun = @(beta,x)(filter([beta(1),beta(2),beta(3),beta(4),beta(5),beta(6)],...
                        [beta(7),beta(8),beta(9),beta(10),beta(11),beta(12)],x));
                    BETA = nlinfit(xx,yy,fun,BETA0);
                    BETA=BETA/BETA(7);
                    B=BETA(1:6); A=BETA(7:end);
                end
            else
                B=filt{1}; A=filt{2};
            end
                
            y(splitI(iSplit)+1:end,iChan)=...
                y(splitI(iSplit)+1:end,iChan)-...
                filter(B,A,step*ones(size(y,1)-splitI(iSplit),1));
        end
    end
end


if ~nargout
    % don't return values, just plot
    disp(['steps at: ', num2str(splitI(2:end-1))]);
    figure(1); clf; nt_banner('nt_destep');
    xx=[x(:),y(:)];
    lim=[min(xx(:)),max(xx(:))]; lim=[lim(1)-0.1*(lim(2)-lim(1)), lim(2)+0.1*(lim(2)-lim(1))];
    subplot 211; plot([x,y]); xlabel('samples'); ylim(lim); legend('raw','clean'); legend boxoff
    subplot 212;     plot(y,'r'); xlabel('samples'); legend('clean'); legend boxoff
    figure(2); clf
    plot(filter(B,A,ones(200,1)),'.-'); xlabel('samples'); title('estimated step response of antialiasing filter');
    clear y, filt;
end



% test code
if 0
    N=8;
    Wn=0.25; % CTF corner frequency is 1/4 of sampling rate
    nsamples=100;
    [B,A] = butter(N,Wn);
    y=filter(B,A,ones(nsamples,1));
    BETA0=[1, zeros(1,8), 1, zeros(1,8)];
    fun = @(beta,x)(filter([beta(1),beta(2),beta(3),beta(4),beta(5),beta(6),beta(7),beta(8),beta(9)],...
        [beta(10),beta(11),beta(12),beta(13),beta(14),beta(15),beta(16),beta(17),beta(18)],x));
    x=ones(nsamples,1);
    BETA = nlinfit(x,y,fun,BETA0);
    BETA=BETA/BETA(10);
    BB=BETA(1:9); AA=BETA(10:end);
    figure(1); clf;
    plot([y,filter(BB,AA,ones(size(y)))])
end

if 0
    N=8;
    Wn=0.25; % CTF corner frequency is 1/4 of sampling rate
    [B,A] = butter(N,Wn);
    x=[zeros(1100,1);ones(2000,1)];
    x=filter(B,A,x);
    x=x+0.01*randn(size(x));
    nt_destep(x);
end
if 0
    N=8;
    Wn=0.25; % CTF corner frequency is 1/4 of sampling rate
    [B,A] = butter(N,Wn);
    x=[zeros(1100,1);ones(1100,1);2*ones(1100,1);3*ones(1100,1);4*ones(1100,1);0*ones(1100,1)];
    x=filter(B,A,x);
    x=x+0.0*randn(size(x));
    nt_destep(x);
end
if 0
    x=ft_read_data('/data/meg/phantom090715_BrainampDBS_20150709_18.ds'); x=permute(x,[2,1,3]); x=x(:,33:306,:);
    x=x(:,100,1);
    nt_destep(x,[],[],30);
end

    
