function y=nt_inpaint(x,w)
%y=nt_inpaint(x,w) - interpolate missing samples of matrix
%
%  y: interpolated data
%
%  x: data to interpolate
%  w: weight matrix
%
% Missing samples (w==0) of a channel are replaced by the projection of
% that channel on the subspace spanned by intact channels, based on
% covariance estimates from intact data.
% 
% See nt_detrend_robust, nt_star.
%
% NoiseTools.
nt_greetings;

% Flexible parameters
PCA_THRESH=10^-25;
minProp=0.5; % minimum proportion of data samples for covariance computation

%{
We estimate the weighted covariance matrix of the data, then project
invalid (weight=0) samples on the subspace spanned by the valid (weight=1)
samples.

The main obstacle is computational, as there are a very large number of
configurations of valid / invalid channels to cater to.  
%}

%% check parameters, check/fix sizes
if nargin<2; error('!'); end

if any(size(w) ~= size(x)); error('w must be of same size as x'); end
if any(w ~= 0 & w ~= 1); error('weights must be 0 or 1'); end
if any(0 == sum(w,2)); warning('for some samples all channels have zero weight'); end
x=nt_unfold(x);
w=nt_unfold(w);

y=x;

for iChan=1:size(x,2)
    if 0; fprintf('Fixing channel %d \n',iChan); end
    
    % Any artifacts ?
    badSamples=~w(:,iChan);
    if all(~badSamples); continue; end
    minDur=sum(w(:,iChan))*minProp;   
           
    % Find best data to compute covariance
    iOtherChannels=setdiff(1:size(x,2),iChan);      % channels on which to project
    iOthersAllGood=all(w(:,[iChan, iOtherChannels]),2);      % good samples common to all other channels
    nGoodPerChannel=sum(w(iOthersAllGood,:));       % number of good samples on each channel
    while sum(iOthersAllGood) < minDur
        [~, iWorst] = min(nGoodPerChannel(iOtherChannels));
        iOtherChannels(iWorst)=[];                  % discard channel with fewest good samples
        iOthersAllGood=all(w(:,[iChan, iOtherChannels]),2);
        nGoodPerChannel=sum(w(iOthersAllGood,:));      
    end    
    x=nt_demean(x,iOthersAllGood);
    mn=y-x; 
    cc=nt_cov(x(iOthersAllGood,:)); % covariance over good samples        
      
    % Copy data to fix
    wBad=w(badSamples,:);
    xBad=x(badSamples,:);
    yBad=y(badSamples,:);
    
    % Get weights with sparse pattern dictionary for interporlation
    wBad(:,~ismember(1:size(x,2),iOtherChannels))=0; % Put weight of unusable channels to 0 to decrease pattern dictionary size.    
    [~, wwSparse,~,iBackSparse]=sparsePatternDict(wBad);
    iBadPatternsSparse=find(~wwSparse(:,iChan));    
    
    % Browse patterns and fix using appropriate channels 
    for iPattern=iBadPatternsSparse'
        iGood=find(wwSparse(iPattern,:) & ismember(1:size(x,2),iOtherChannels)); % indices of channels to use
        inPattern=iBackSparse==iPattern; % indices of samples to fix  
        
        [V,D]=eig(cc(iGood,iGood)); V=real(V); D=real(D);
        topcs=V(:,D/max(D) > PCA_THRESH); % discard weak dims 
        b=cc(iChan,iGood)*topcs/(topcs'*cc(iGood,iGood)*topcs); % projection matrix  
        xx=(xBad(inPattern,iGood));
        yBad(inPattern,iChan)=xx*(topcs*b')+mn(inPattern,iChan); % projection 
    end % of bad patterns loop
    
    % Inject fixed data back
    y(badSamples,:)=yBad;
        
end % of channel loop

% Create parcimonious pattern dictionary
function [w, ww,nOccurrences,iBack]=sparsePatternDict(w)

%Parameters
ratio1=1/100; % maximum iteration ratio
ratio2=1/100; % minimum efficiency ratio
minThresh=size(w,2)/3; % minimum number of good channels to keep in each pattern

% Initialize before while loop
[ww, nOccurrences, iBack]=patternDict(w);
iSimPatterns=1:size(ww,1);
maxIterNum=size(ww,1)*ratio1; % maximum number of iterations allowed
minEff=size(ww,1)*ratio2; % minimum efficiency required: number of grouped pattern / dictionary size
iIter=1;

% Group patterns until it has taken too much time or is not efficient enough
while iIter<=maxIterNum && numel(iSimPatterns)> minEff 
    
    % Compute similarity metric
%     sim=2*(ww-0.5)*2*(ww'-0.5); % number of common bad channels AND common good channels
    sim=ww*ww'; % number of common good channels only
    sim_noDiag=reshape(sim(~logical(diag(ones(size(ww,1),1)))),size(ww,1)-1,size(ww,1)); % values excluding diagonal
    
    % Compute threshold for number of common good channels across grouped
    % patterns
    weight=(iIter-1)/maxIterNum;
    nThresh=max(minThresh,...
        (mean((1+weight)*sim_noDiag(:))+(1-weight)*max(sim_noDiag(:)))/2);

    % Pick the pattern with the highest number of similar patterns
    [~, iPat]=max(sum(sim>=nThresh));  
 
    % Find most similar patterns
    iSimPatterns=find(sim(iPat,:)>=nThresh); % indices of similar patterns
    
    % Change all to unique pattern with common good channels only
    inPat=ismember(iBack, iSimPatterns);
    w(inPat,:)=repmat(all(ww(iSimPatterns,:),1),sum(inPat),1); 
    
    % Update pattern dictionary
    [ww,nOccurrences,iBack]=patternDict(w); 
    if 0; fprintf('%dth iteration, nDiffThresh=%g, %d patterns regrouped, %d patterns in dictionary\n',...
        iIter, nThresh, numel(iSimPatterns), size(ww,1)); end
    
    % Update other parameters
    maxIterNum=min(maxIterNum,size(ww,1)*ratio1+iIter); % maximum number of iterations allowed
    minEff=size(ww,1)*ratio2; % minimum efficiency required: number of grouped pattern / dictionary size
    iIter=iIter+1;    
    
end % of while loop

% create a dictionary of weight patterns
function [ww,nOccurrences,iBack]=patternDict(w)
% ww: dictionary of patterns
% nOccurrences: number of times each pattern occurred
% iBack: index to reconstruct input from dictionary
[ww,~,IC,nOccurrences]=nt_unique(w,'rows');
[nOccurrences,iSort]=sort(nOccurrences, 'descend'); % sort by decreasing number
[~,iReverse]=sort(iSort); % 
ww=ww(iSort,:); % same order for patterns, w = ww(iReverse1(IC),:)
iBack=iReverse(IC); % w = ww(iBack,:)

%%% TEST %%%
if 0
    x0=sin(2*pi*(1:10000)'*(1:10)/10000);
    x=x0*randn(10,20)+1;
    x(1:4000,1)=x(1:4000,1)+0.3*randn(size(x(1:4000,1)));
    w=ones(size(x));  
    w(1:4000,1)=0; w(4001:6000,3)=0; w(6000:7000,4)=0; w(7000:8000,5)=0; w(8000:9000,6)=0; w(9000:10000,7)=0;
    %w=randn(size(w))>0;
    %b=nt_regw(x(:,1),x(:,2:end),w(:,1)); y=x(:,2:end)*b;
    y=nt_inpaint(x,w);
    figure(2); clf
    subplot 211; plot([x(:,1),y(:,1)]); legend('raw','clean'); title('one channel'); subplot 212; plot(x(:,1)-y(:,1)); title('raw-clean');
end
if 0
    x=randn(10000,10);
    w=ones(size(x));
    y=nt_inpaint(x,w);
    figure(1); clf
    subplot 311; plot(x); title('raw');  subplot 312; plot(y); title('clean'); subplot 313; plot(x-y); title('raw-clean');
end
if 0
    x0=sin(2*pi*(1:10000)'*(1:10)/10000);
    x=x0*randn(10,20)+1;
    w=ones(size(x));
    x(1:1000,1:2)=100; w(1:1000,1:2)=0;
    y=nt_inpaint(x,w);
    figure(1); clf
    subplot 311; plot(x); title('raw');  subplot 312; plot(y); title('clean'); subplot 313; plot(x-y); title('raw-clean');
end
if 0
    %[p,x]=nt_read_data('/data/meg/theoldmanandthesea/eeg/mc/MC_aespa_speech_45.mat'); x=x'; x=x(:,1:128); x=x(1:10000,:);
    [p,x]=nt_read_data('/data/meg/arzounian/ADC_DA_140521_p20/ADC_DA_140521_p20_01_calib'); x=x'; x=x(1:10000,:);
    
    x=nt_demean(x);
    [x,w]=nt_detrend_robust(x,10);   
    profile on; y=nt_inpaint(x,w); profile report;
    figure(1); clf
    subplot 311; plot(x); title('raw');  subplot 312; plot(y); title('clean'); subplot 313; plot(x-y); title('raw-clean');
    figure(2); clf
    ch=35;subplot 311; plot([x(:,ch),y(:,ch)]); subplot 312; plot(x(:,ch)-y(:,ch)); subplot 313; plot(w(:,ch), '.-');
end


