function [topcs,pwr,y]=nt_pca_big(x,N,aggregate,normflag)
%[topcs,pwr,y]=nt_pca_big(x,N,normflag) - approximate PCA of big data
%
%  r: matrix to project to PCs
%  pwr: power per PC
%  y: PCs
%
%  x: data
%  N: chunk size [default: 100]
%  aggregate: 'local' [default], 'global', or 'globmean'
%  normflag: normalize at each level [default: don't]
%
% Data are processed recursively in chunks of N columns.  Each chunk is
% submitted to PCA and either N/2 PCs are retained ('local') or 1 PC is
% retained ('global').  If 'globmean' the data are averaged over chunks 
% of nchans/N channels, then PCA
%
% NoiseTools
nt_greetings();

if nargin<1; error('!'); end
if nargin<2||isempty(N); 
    N=100;
    disp('defaulting to N=100');
end
if nargin<3||isempty(aggregate); aggregate='local'; end
if nargin<4||isempty(normflag); normflag=0; end

[nsamples,nchans,ntrials]=size(x); 
x=nt_unfold(x);

if normflag;
    tonorm=1./sqrt(mean(x.^2)); 
    tonorm(find(isnan(tonorm)))=0;
    x=bsxfun(@times,x,tonorm);
end
if ~ (strcmp(aggregate,'local') || strcmp(aggregate,'global') || strcmp(aggregate,'globmean'))
    error('!');
end



if nchans<=N; 
    [topcs,pwr]=nt_pca0(x);
else
    if strcmp(aggregate,'local')
        topcs=zeros(nchans,N);
        a=round(nchans/2);
        NN=round(N/2);
        tp=nt_pca_big(x(:,1:a),N,aggregate,normflag);
        [topcs(1:a,1:NN)]=tp(:,1:NN);
        tp=nt_pca_big(x(:,a+1:end),N,aggregate,normflag);
        [topcs(a+1:end,NN+1:N)]=tp(:,1:(N-NN));
        [topcs2,pwr,z]=nt_pca_big(x*topcs,N,aggregate,normflag);
        topcs=topcs*topcs2;
    elseif strcmp(aggregate,'global') 
        nChunks=ceil(nchans/(N));
        nKeep=ceil(N/nChunks);
        topcs=zeros(nchans,nChunks*nKeep);
        for iChunk=1:nChunks
            a=(iChunk-1)*N;
            idx=(a+1):min(a+N,nchans);
            tp=nt_pca_big(x(:,idx),N,aggregate,normflag);
            topcs(idx,(iChunk-1)*nKeep+(1:nKeep))=tp(:,1:nKeep);
        end
        %figure(1); clf; nt_imagescc(topcs); pause
        [topcs2,pwr,z]=nt_pca_big(x*topcs,N,aggregate,normflag);
        topcs=topcs*topcs2;
    else % globmean
        nChunks=ceil(nchans/N);
        topcs=zeros(nchans,N);
        for iChunk=1:nChunks
            a=(iChunk-1)*N;
            idx=(a+1):min(a+N,nchans);
            topcs(idx,iChunk)=1/numel(idx);
            %disp(idx)
        end
        %figure(1); clf; nt_imagescc(topcs); pause
        %[topcs2,pwr,z]=nt_pca_big(x*topcs,N,'local',normflag);
        [topcs2,pwr,z]=nt_pca0(x*topcs);
        topcs=topcs*topcs2;
    end
        
end
if ~strcmp(aggregate,'globmean') 
    if size(topcs,2)>=N; 
        topcs=topcs(:,1:N); % truncate to save space
        pwr=pwr(1:N);
    else
        topcs(:,end:N)=0; % should rarely happen
        pwr(end:N)=0;
    end
end

%disp([size(topcs)])

if normflag; topcs=bsxfun(@times,topcs,tonorm'); end

if nargout>1; y=x*topcs; y=nt_fold(y,nsamples);end

if ~nargout % don't return, just plot something
    y=nt_unfold(y);
    semilogy(mean(y.^2), '.-');
    xlabel('pc'); ylabel('power');
    figure(2); clf
    plot(y); xlabel('sample');
    figure(3); clf
    nt_imagescc(nt_normcol(x'*y)'); 
    ylabel('pc'); xlabel('channel');
    clear topcs y
end






%% test code
if 0
    % basic
    x=randn(10000,3000);
    N=20; % blocksize
    [topcs,pwr,y]=nt_pca_big(x,N,'local');
    figure(1); clf; plot(pwr);
    [topcs,pwr,y]=nt_pca_big(x,N,'global');
    figure(2); clf; plot(pwr);
    [topcs,pwr,y]=nt_pca_big(x,N,'globmean');
    figure(3); clf; plot(pwr);
end
if 0 
    x=sin(2*pi*3*(1:1000)'/1000)*randn(1,1000);
    x=2*x + randn(size(x));
    N=30; 
    [topcs,pwr,y]=nt_pca_big(x,N);
    figure(1); plot(pwr);
    figure(2); subplot 121; plot(x); subplot 122; plot(x*topcs);
end   
if 0 
    nchans=100000;
    nnoise=900;
    x=repmat(sin(2*pi*3*(1:1000)'/1000), 1, nchans);
	x= 0.06*nt_normcol(x) + nt_normcol(randn(size(x,1),nnoise)*randn(nnoise,nchans));
    N=200;
    [topcs,pwr,y]=nt_pca_big(x,N,'global');
    figure(1); plot(pwr);
    figure(2); subplot 121; plot(x(:,1:100)); subplot 122; plot(y(:,1));
end   
if 0 
    nchans=100000;
    nnoise=900;
    x=repmat(sin(2*pi*3*(1:1000)'/1000), 1, nchans);
	x= 0.01*nt_normcol(x) + nt_normcol(randn(size(x,1),nnoise)*randn(nnoise,nchans));
    [topcs,pwr,y]=nt_pca_big(x,100,'globmean');
    figure(1); plot(pwr);
    figure(2); subplot 121; plot(x(:,1:100)); subplot 122; plot(y(:,1));
end   

